options(stringsAsFactors = F)
library(seqinr)
library(gtools)
args <- commandArgs(TRUE)
refFile<-args[1]
contigsFile<-args[2]
tableFile<-args[3]
refseq<-args[4]

#Import ref
ref<-as.character(read.fasta(refFile)[[1]])

#Import contigs and convert them to vectors
contigs.i<-read.fasta(contigsFile)
contigs<-list()
for (i in 1:length(contigs.i)){
  contigs[[i]]<-as.character(contigs.i[[i]])
}
names(contigs)<-names(contigs.i)

#Import alignment table
table <- read.delim(tableFile, header=FALSE)
table<-table[mixedorder(table[,2]),]
if(all(table$V6==0)){
  table<-table  
} else {
  table<-table[-c(which(table$V6!=0)),]
}

#Remove unaligned contigs
contigs<-contigs[names(contigs.i)%in%table$V2]

#Add dashes according to alignment position
contigs.f<-list()
for (i in 1:nrow(table)){
  contigs.f[[i]]<-c(rep("-",table[i,7]-1),contigs[[which(names(contigs)==table$V2[i])]][table[i,9]:table[i,10]],rep("-",length(ref)-table[i,8]))
}

#Change the sense of those aligned in the other direction
complementary<-cbind.data.frame(n=c("a","t","c","g"),c=c("t","a","g","c"))
for (a in which(table[,10]<table[,9])){
  contigs.f[[a]]<-unlist(lapply(1:length(contigs.f[[a]]), function(i) 
    ifelse(contigs.f[[a]][i]==complementary[1,1],complementary[1,2],
           ifelse(contigs.f[[a]][i]==complementary[2,1],complementary[2,2],
                  ifelse(contigs.f[[a]][i]==complementary[3,1],complementary[3,2],
                         ifelse(contigs.f[[a]][i]==complementary[4,1],complementary[4,2],
                                contigs.f[[a]][i]))))))
}

#Create a df with the alignment
alignment<-rbind.data.frame(ref,do.call(rbind.data.frame,contigs.f))
colnames(alignment)<-c(1:ncol(alignment))
names<-c("ref",1:nrow(table))

#Consensus of alignment
consensus<-unlist(lapply(1:ncol(alignment), function(i) 
  ifelse(length(unique(alignment[2:nrow(alignment),i][alignment[2:nrow(alignment),i]!="-"]))==1,unique(alignment[2:nrow(alignment),i][alignment[2:nrow(alignment),i]!="-"]),
         ifelse(length(unique(alignment[2:nrow(alignment),i][alignment[2:nrow(alignment),i]!="-"]))==0,"-",
                ifelse((max(table(alignment[2:nrow(alignment),i][alignment[2:nrow(alignment),i]!="-"]))>1),
                       names(table(alignment[2:nrow(alignment),i][alignment[2:nrow(alignment),i]!="-"]))[unname(which(table(alignment[2:nrow(alignment),i][alignment[2:nrow(alignment),i]!="-"])==max(table(alignment[2:nrow(alignment),i][alignment[2:nrow(alignment),i]!="-"]))))],
                       "?")))))

sample=strsplit(strsplit(contigsFile,"/")[[1]][2],"[.]")[[1]][1]
iteration=strsplit(contigsFile,"[.]")[[1]][3]
write.fasta(list(consensus),"consensus",paste("Assembly/",sample,"/",refseq,"/consensus/",sample,".",iteration,".consensus.fa",sep=""),nbchar=60)

